/*
 * Programm:	ChgCol
 * Datei:	---
 * Version:	1.0
 * Datum:	28.11.1993
 * Autor:	Klaus Rotter, Linsenfeld 15, 8510 Frth
 * Computer:	Amiga 500 mit Kickstart 2.0
 * Erweiterung: ----
 * Sprache:	Ansi-C
 * Compiler:	DICE 2.07.56r
 * Copyright:	it's in the Public Domain
 * Bemerkung:
 */

#include <stdio.h>

#include <proto/intuition.h>
#include <proto/graphics.h>

#include <intuition/intuitionbase.h>

int print(char *str)
{
    Write(Output(),str,strlen(str));
    return(0);
}

struct	IntuitionBase *IntuitionBase;
struct	GfxBase       *GfxBase;

void end(int dum);
{
    if (IntuitionBase)  CloseLibrary(IntuitionBase);
    if (GfxBase)        CloseLibrary(GfxBase);
    exit(0);
}


char HelpTxt =
    "This is ChgCol V1.0, written "__DATE__" by Klaus Rotter (klaus@touch.nbg.sub.org)\n"
    "It is in the Public Domain\n"
    "Usage: ChgCol ? -? -h -n -d -c COL1 COL2 COL3 ...\n"
    "? -? -h: Prints out this help message\n"
    "-n %s  : Screen-name of screen to change colors\n"
    "-d %d  : Time in seconds before change of colors\n\n";

void main(int argc, char **argv)
{
    short   a,i;
    long    color;
    char    *dummy;

    ULONG   ilock;
    struct  ViewPort *VP;
    struct  Screen   *MyScreen = NULL;

    if (!(IntuitionBase = (struct IntuitionBase *) OpenLibrary("intuition.library",0L)))
	end(print("Can't open intuition.library\n"));

    if (!(GfxBase = (struct GfxBase *) OpenLibrary("graphics.library",0L)))
	end(print("Can't open graphics.library\n"));

    for (i=1; i<argc; i++)
    {
	// Put Help Msg

	if (argc == 1 || !strcmp(argv[i],"?") || !strcmp(argv[i],"-?") || !strcmp(argv[i],"-h"))
	    print("HelpTxt");

	if (!strcmp(argv[i],"-d"))
	{
	    long du = abs(strtol(argv[i++]));

	    printf("Waiting: %d seconds\n",du*50);

	    if (du > 3600)
	    {
		print("Ah... do you really want to wait longer than an hour ?\nMaybe you, but I do not...\n");
		end();
	    }
	    Delay(50*du);   // Let me stand here for a while...
	}

	if (!strcmp(argv[i],"-n"))
	{
	    ilock = LockIBase(0);

	    MyScreen = IntuitionBase->FirstScreen;
	    i++;

	    while (strcmp(MyScreen->Title,argv[i]))
	    {
		if (!MyScreen = MyScreen->NextScreen)
		{
		    UnlockIBase(ilock);

		    print("Screen '");
		    print(argv[i]);
		    print("' not found\n");
		    end();
		}
	    }
	    UnlockIBase(ilock);

	    printf("Screen %s at address 0x%08x\n",argv[i],MyScreen);
	}


	if (!strcmp(argv[i],"-c"))  // ok... start witch coloring
	{
	    if (MyScreen == NULL)
	    {
		ilock = LockIBase(0);

		MyScreen = IntuitionBase->ActiveScreen;
		UnlockIBase(ilock);
	    }

	    a = 0;
	    i++;

	    VP = &(MyScreen->ViewPort);


	    for (; i<argc; i++)
	    {
		if (argv[i] != "NONE")
		{
		    color = strtol(argv[i],&dummy,0);
		    if (color > 0x0FFF || color<0)
		    {
			print("Color not in range: ");
			print(argv[i]);
			print("\n");
		    }
		    else
			SetRGB4(VP, a++,(color & 0xF00)>>8, (color & 0x0F0)>>4,color & 0x00F);
		}
	    }
	    end();
	}
    }
}
