/*************************************************************************
 *
 * PROGRAM	Compiler.c
 * PART
 * VERSION	0.1
 * DATE 	20/08/91
 * CHANGED
 * AUTHOR	Aho,Sethi,Ullman: COMPILERS, Page 89-84
 * ADDRESS
 * COMPUTER	Amiga 500 with Kickstart 1.3 and 2.0
 * EXTENSION
 * LANGUAGE	ANSI-C
 * COMPILER	Aztec-C 5.0
 * COPYRIGHT	PD
 * COMMENT	Amiga port by Klaus Rotter
 *
 * HISTORY
 *
 *************************************************************************/

/**** proto.h ************************************************************/

int lexan(void);
void parse(void);
void expr(void);
void term(void);
void expow(void);
void factor(void);
void match(int t);
void emit(int t, int tval);
int lookup(char s[]);
int insert(char s[], int tok);
void init(void);
int error(char *m);


/**** global.h ***********************************************************/

#include <stdio.h>
#include <ctype.h>

#define TRUE	1
#define BSIZE	128
#define NONE	 -1
#define EOS	'\0'

#define NUM	256
#define DIV	257
#define MOD	258
#define ID	259
#define FC	260
#define DONE	261

extern int tokenval;
extern int lineno;

struct entry {
    char *lexptr;
    int  token;
};

extern struct entry symtable[];

/**** lexer.c ***********************************************************/

/* include global.h */ /* fr jedes Modul */

char lexbuf[BSIZE];
int  lineno = 1;
int  tokenval = NONE;

int lexan(void)
{
    int t;
    while (TRUE) {
	t = getchar();
	if (t == ' ' || t == '\t'); /* entfernen von Leerrumen */
/*
	else if (t == '\n')
	    lineno += 1;
*/	    
	else if (isdigit(t)) {
	    ungetc(t,stdin);
	    scanf("%d",&tokenval);
	    return (NUM);
	}
	else if (isalpha(t)) {
	    int p,b = 0;
	    while (isalnum(t)) {
		lexbuf[b] = t;
		t = getchar();
		b += 1;
		if (b >= BSIZE)
		    error("Compilerfehler: lexan");
	    }
	    lexbuf[b] = EOS;

	    p = lookup(lexbuf);
	    if (p == 0) {
		if (t == '(')
		    p = insert(lexbuf,FC);
		else
		    p = insert(lexbuf,ID);
	    }

	    if (t != EOF)
		ungetc(t,stdin);

	    tokenval = p;
	    return (symtable[p].token);
	}
	else if (t == EOF)
	    return DONE;
	else {
	    tokenval = NONE;
	    return (t);
	}
    }
}

/**** parser.c *********************************************************/

/* #include "global.h" */

int lookahead;

void parse(void)
{
    lookahead = lexan();
    while (lookahead != DONE) {
	expr();
	match('\n');
    }
}

void expr(void)
{
    int t;
    term();
    while (TRUE) {
	switch (lookahead) {
	    case '+':
	    case '-':
		t = lookahead;
		match (lookahead);
		term();
		emit(t,NONE);
		continue;
	    default:
		return;
	}
    }
}

void term(void)
{
    int t;
    expow();
    while (TRUE) {
	switch (lookahead) {
	    case '*':
	    case '/':
	    case DIV:
	    case MOD:
		t = lookahead;
		match(lookahead);
		expow();
		emit(t,NONE);
		continue;
	    default:
		return;
	}
    }
}

void expow(void)
{
    int t;
    factor();
    while (TRUE) {
	if (lookahead == '^') {
	    t = lookahead;
	    match(lookahead);
	    factor();
	    emit(t,NONE);
	    continue;
	}
	else return;
    }
}

void factor(void)
{
    int t;

    switch (lookahead) {
	case '(':
	    match('(');
	    expr();
	    match(')');
	    break;
	case FC:
	    t = tokenval;
	    match(FC);
	    factor();
	    emit(FC,t);
	    break;
	case NUM:
	    emit(NUM,tokenval);
	    match(NUM);
	    break;
	case ID:
	    emit(ID,tokenval);
	    match(ID);
	    break;
	default:
	    error("Syntaxfehler: factor");
    }
}

void match(int t)
{
    if (lookahead == t)
	lookahead = lexan();
    else
	error("Syntaxfehler: match");
}

/*
long sp2hex(float ft)
{
    long t, *a;

    a = &ft;
    t = *a;

    return(t);
}
*/

/**** emitter.c ********************************************************/

/* #include "global.h" */

double stack[1000];
int    pos = 0;

void emit(int t, int tval)
{
    switch (t) {
	case '+':
	    stack[pos-2] = stack[pos-2] + stack[pos-1]; 
	    pos--;
	    printf("ADD\n");
	    break;
	case '-':
	    stack[pos-2] = stack[pos-2] - stack[pos-1]; 
	    pos--;
	    printf("SUB\n");
	    break;
	case '*':
	    stack[pos-2] = stack[pos-2] * stack[pos-1]; 
	    pos--;
	    printf("MUL\n");
	    break;
	case '/':
	    stack[pos-2] = stack[pos-2] / stack[pos-1]; 
	    pos--;
	    printf("DIV\n");
	    break;
	case '^':
	    stack[pos-2] = pow(stack[pos-2],stack[pos-1]); 
	    pos--;
	    printf("POW\n");
	    break;
/*
	case DIV:
	    printf("DIV\n");
	    break;
	case MOD:
	    printf("MOD\n");
	    break;
*/
	case NUM:
	    printf("%d\n",tval);
	    stack[pos++] = (double) tval;
	    break;
	case ID:
	    printf("PUT %s\n",symtable[tval].lexptr);
	    break;
	case FC:
	    printf("CALL %s\n",symtable[tval].lexptr);
	    break;
	default:
	    printf("token %d, tokenval %d\n",t,tval);
    }
}

/**** symbol.c *********************************************************/

/* include "global.h */

#define STRMAX 999
#define SYMMAX 100

char lexemes[STRMAX];
int  lastchar = -1;
struct entry symtable[SYMMAX];
int lastentry = 0;

int lookup(char s[])
{
    int p;
    for (p = lastentry; p>0; p--)
	if (strcmp(symtable[p].lexptr,s) == 0)
	    return p;
    return (0);
}

int insert(char s[], int tok)
{
    int len;
    len = strlen(s);
    if (lastentry+1 >= SYMMAX)
	error("Symboltabelle voll");
    if (lastchar + len + 1 >= STRMAX)
	error("Lexemarry voll");
    lastentry += 1;
    symtable[lastentry].token = tok;
    symtable[lastentry].lexptr = &lexemes[lastchar + 1];
    lastchar += len + 1;
    strcpy(symtable[lastentry].lexptr,s);
    return (lastentry);
}


/**** init.c ***********************************************************/

/* include "global.h */

struct entry keywords[] = {
    "div",DIV,
    "mod",MOD,
    0,0
};

void init(void)
{
    struct entry *p;
    for (p = keywords; p->token; p++)
	insert(p->lexptr,p->token);
}


/**** error.c **********************************************************/

/* include "global.h */

int error(char *m)
{
    printf("Zeile %d: %s\n",lineno,m);
    exit(1);
}

/**** main.c ***********************************************************/

/* include "global.h */

int main(void)
{
    int i;

    printf("This is a simple one-pass-compiler Version 0.1\nsee: Aho,Sehti,Ullman: Compilers, (The Dragon's Book)\n");
    printf("Amiga Port by Klaus Rotter\n");
    init();
    parse();
    printf("\nSymtable: %d entries  Stringtable: %d chars\n",lastentry,lastchar+1);

    for (i=1; i<=lastentry; i++)
	printf("Entry %d:  Cont %s:  Token %d\n",i,symtable[i].lexptr,symtable[i].token);

    for (i=0; i<pos; i++)
        printf("Stack[%d] = %g\n",i,stack[i]);

    exit(0);
}

/***********************************************************************/
