/*************************************************************************
 *
 * PROGRAM	Mandelbrot
 * PART
 * VERSION	0.2
 * DATE 	10/05/89
 * CHANGED	12/12/92
 * AUTHOR	Klaus Rotter
 * ADDRESS	Linsenfeld 15, W-8510 Fuerth, Germany
 * EMAIL	Internet: k.rotter@ame.nbg.sub.org
 *		Z-Netz:   k.rotter@ame.zer
 * COMPUTER	Amiga with OS 2.04
 * EXTENSION
 * LANGUAGE	ANSI-C
 * COMPILER	DICE 2.06
 * COPYRIGHT	(c) Copyright 1992 by Klaus Rotter
 * COMMENT
 *
 * HISTORY	10/05/92 Progrmm unter Kick 1.3 in Aztec-C
 *		12/12/92 Kick 2.0, Compiler DICE
 *
 *************************************************************************/

#define X_MAX 320
#define Y_MAX 255
#define MAX_FARBEN 31  /* 0 wird nicht verwendet */

#include <intuition/intuition.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>

struct	Screen	      *Scr;
struct	IntuitionBase *IntuitionBase;
struct	GfxBase       *GfxBase;
struct	Window	      *Win;
struct	RastPort      *Rp;
struct	ViewPort      *Vp;

float	delta_x, delta_y, rmax, rmin, imax, imin;
int	maxit;

#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/exec_protos.h>

static short color[] = {
    0x000, 0x03F, 0x05F, 0x08F, 0x0AF, 0x0DF, 0x0FE, 0x0F8, 0x0F6,
    0x0F3, 0x0F0, 0x3F0, 0x5F0, 0x8F0, 0x0F0, 0x2F0, 0x5F0, 0x5C0,
    0x5A0, 0x590, 0xF50, 0xF20, 0xF00, 0xF04, 0xF08, 0xF09, 0xF0C,
    0xF0F, 0xB0F, 0x70F, 0x40F, 0x00F
};

struct	IntuitionBase *IntuitionBase;
struct	GfxBase       *GfxBase;

void ende(void)
{
    if (Win)      CloseWindow(Win);
    if (Scr)      CloseScreen(Scr);

    if (IntuitionBase) CloseLibrary(IntuitionBase);
    if (GfxBase)       CloseLibrary(GfxBase);
    exit(0);
}

void main(void)
{
    short   zeile, spalte, farbe;
    float   cr = MIN_R, ci = MAX_I, zr, zi, i;
    double  rmin_in, rmax_in, imin_in, imax_in;
    float   rmin, imin_in;

    short   it,maxit;
    char    *cia = (char *) 0xBFE001;
    char    reg;

    struct Screen   *Scr;
    struct RastPort *Rp;
    struct ViewPort *Vp;

    IntuitionBase = OpenLibrary("intuition.library",37);
    GfxBase	  = OpenLibrary("graphics.library",37);

    if (!IntuitionBase) {
	puts("Program requires AmigaOS 2.04 or higher");
	ende();
    }

    printf("\n Mandelbrot-Menge (Apfelmnnchen) Programm V0.2 ");
    printf("\n geschrieben 1989 (updated 1992) von Klaus Rotter");
    printf("\n\n Bitte geben Sie den Ausschnitt der Komplexen Ebene ein");
    printf("\n\n Realteil Min.       : "); fflush(stdin);
    scanf("%lf",&rmin);
    printf(" Realteil Max.       : ");
    scanf("%lf",&rmax);
    printf(" Imaginrteil Min.   : ");
    scanf("%lf",&imin);
    printf(" Imaginrteil Max.   : ");
    scanf("%lf",&imax);
    printf(" Wieviele Iterationen: ");
    scanf("%d",&maxit);

    // Kurze Art und Weise, unter OS2.04 einen Screen zu oeffnen

    Scr = OpenScreenTags(NULL, SA_Depth, 5, TAG_END);

    if (!Scr) {
	puts("Program can't open screen");
	ende();
    }

    Win = OpenWindow Tags(NULL, WA_CustomScreen, Scr,
			  WA_CloseGadget, WA_Borderless, TAG_END);
    if (!IntuitionBase) {
	puts("Program can't open window");
	ende();
    }

    Rp= &(Scr->RastPort);
    Vp= &(Scr->ViewPort);

    LoadRGB4(Vp,&color[0],32);

    // Dies ist nun der eigentliche Algorithmus, der die Mandelbrot
    // Menge bestimmt.

    rmin = (float) rmin_in;
    imin = (float) imin_in;

    cr = rmin;
    ci = imax;

    delta_x = (float)(rmax_in-rmin_in)/ (double) X_MAX;
    delta_y = (float)(imax_in-imin_in)/ (double) Y_MAX;

    for (zeile=0; zeile<=MAX_Y; zeile++)
    {
	for (spalte=0; spalte<=MAX_X; spalte++)
	{
	    it = 0;
	    zr = 0;
	    zi = 0;

	    while ((it < MAX_IT) && ((zr*zr + zi*zi) < 4))
	    {
		i = zi;
		zi = 2*i*zr-ci;
		zr = zr*zr-i*i-cr;
		it++;
	    }
	    farbe = (it % MAX_FARBEN)+1;
	    if (it >= MAX_IT) farbe = 0;
	    SetAPen(Rp,farbe);
	    WritePixel(Rp,spalte,zeile);
	    cr += DELTA_X;

	    reg = *cia;
	    if (!(reg & 64))   // Soll's kurz sein, so braucht man ein goto
		goto ende;  // sorry...
	}
	cr = MIN_R;
	ci -= DELTA_Y;
    }
ende:
    CloseScreen(Scr);
    CloseLibrary(IntuitionBase);
    CloseLibrary(GfxBase);
}
