/*************************************************************************
 *
 * PROGRAM	bezier
 * PART
 * VERSION	V0.1
 * DATE 	26/09/93
 * CHANGED
 * AUTHOR	Klaus Rotter
 * ADDRESS	Linsenfeld 15, 8510 Fuerth, Germany
 * EMAIL	klaus@touch.nbg.sub.org
 * COMPUTER	Amiga (OS 2.04 required)
 * EXTENSION
 * LANGUAGE	ANSI-C
 * COMPILER	DICE 2.06
 * COPYRIGHT	(c) Copyright 1992 by Klaus Rotter
 * COMMENT
 *
 * HISTORY
 *
 *************************************************************************/

#define XOFF 320
#define YOFF 200
#define _SP_FLOAT 1
#define PI 3.14159265358

#include <math.h>
#include <libraries/mathieeesp.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/exec_protos.h>

struct IntuitionBase *IntuitionBase = NULL;
struct GfxBase *GfxBase = NULL;
struct Rastport *Rp = NULL;
struct Window *Win = NULL;

extern void quit(void);
extern void main(void);
extern void exit(int);
extern void puts(char *);

void quit(void)
{
    if (IntuitionBase) CloseLibrary(IntuitionBase);
    if (GfxBase)       CloseLibrary(GfxBase);
    if (Win)           CloseWindow(Win);
    exit(0);
}

void main(void)
{
    struct IntuiMsg *msg = NULL;

    IntuitionBase = OpenLibrary("intuition.library",37);
    GfxBase = OpenLibrary("graphics.library",37);

    if (!IntuitionBase || !GfxBase)
    {
	puts("Program requires AmigaOS 2.04 or higher");
	quit();
    }

    Win = OpenWindowTags(NULL,
	WA_Left, 20,
	WA_Top,  20,
	WA_Width, 620,
	WA_Height, 300,
	WA_IDCMP, IDCMP_CLOSEWINDOW|IDCMP_MENUPICK,
	WA_Flags, WFLG_DEPTHGADGET|WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_SIZEGADGET,
	WA_Title, "Math-demo: 3d-functions (created Dez 92 by Klaus Rotter)",
	TAG_END);

    if (!Win)
    {
	puts("Program can't open window");
	quit();
    }

    Rp = Win->RPort;

    SetAPen(Rp,1);

/*
** BEZIER DOCUMENT-STYLE OPTION - released 17 December 1985
**    for LaTeX version 2.09
** Copyright (C) 1985 by Leslie Lamport
**  In a picture environment, writing
**
**	 \bezier{N}(AX,AY)(BX,BY)(CX,CY)
**
**  plots a quadratic Bezier curve from (AX,AY) to (CX,CY), with (BX,BY)
**  as the third Bezier point, using N+1 equally spaced points.
**
**  \bezier{N}(AX,AY)(BX,BY)(CX,CY) ==
**    BEGIN
**	@sc  := N
**	@scp := @sc+1
**	\@xb := 2 * (BX - AX) * \unitlength
**	\@xa := ((CX-AX)*\unitlength - \@xb)/@sc
**	\@yb := 2 * (BY - AY) * \unitlength
**	\@ya := ((CY-AY)*\unitlength - \@yb)/@sc
**	\@pt := square rule of width \@wholewidth
**	@t := 0
**	WHILE @t < @scp
**	  DO  \@x := ((@t*\@xa + @xb) / @sc) * t
**	      \@y := ((@t*\@ya + @yb) / @sc) * t
**	      plot pt with relative coords (\@x,\@y)
**	      @t := @t+1
**	  OD
*/
    long  n, scp, t, xabs, yabs, dx, dy;

    long  ax,ay,bx,by,cx,cy;
    float sc, xb, xa, yb, ya;

    n = 50;
    ax = 20;
    ay = 20;
    bx = 40;
    by = 250;
    cx = 500;
    cy = 280;

    sc = n;
    scp = sc + 1;
    xb = 2 * (bx - ax);
    xa = ((cx - ax) - xb) / sc;
    yb = 2 * (by - ay);
    ya = ((cy - ay) - yb) / sc;

    t = 0;
    xabs = ax;
    yabs = ay;

    Move(Rp,xabs,yabs);

    while (t < scp)
    {
	dx = ((t * xa + xb) / sc) * t;
	dy = ((t * ya + yb) / sc) * t;
	Draw(Rp,xabs+dx,yabs+dy);
	t++;
    }

    msg = WaitPort(Win->UserPort);
    msg = GetMsg(Win->UserPort);
    ReplyMsg(msg);
    quit();
}
