/*************************************************************************
 *
 * PROGRAM	funcomp.c
 * PART
 * VERSION	0.1
 * DATE 	20/08/91
 * CHANGED	23/11/94
 * AUTHOR	Klaus Rotter, very similar to Aho,Sethi,Ullman: COMPILERS, Page 89-84
 * ADDRESS
 * COMPUTER	Linux
 * EXTENSION
 * LANGUAGE	ANSI-C
 * COMPILER	GCC 2.5.8
 * COPYRIGHT	PD
 * COMMENT	
 *
 * HISTORY	20/08/91 Amiga Version
 *		23/11/94 Ported and extented to Linux
 *
 *************************************************************************/

extern int lexan(void);
extern void parse(void);
extern void expr(void);
extern void term(void);
extern void pow(void);
extern void factor(void);
extern void match(int t);
extern void emit(int t, int tval);
extern int lookup(char s[]);
extern int insert(char s[], int tok);
extern void init(void);
extern int error(char *m);
extern void SymOut(void);

extern char line[200];
extern int getbyte(void);
extern void ungetb(int t);

#include <stdio.h>
#include <ctype.h>

#define TRUE	1
#define BSIZE	128
#define NONE	 -1
#define EOS	'\0'

#define NUM	256
#define DIV	257
#define MOD	258
#define ID	259
#define FC	260
#define DONE	261

extern int tokenval;

struct entry {
    char *lexptr;
    int  token;
};

extern struct entry symtable[];

char lexbuf[BSIZE];
int  lineno = 1;
int  tokenval = NONE;

int lexan(void)
{
    int t;
    int p,b = 0;

    while (TRUE) {
	t = getbyte();
	if (t == ' ' || t == '\t'); /* entfernen von Leerrumen */
	else if (isdigit(t)) {
	    while (isdigit(t)) {
		lexbuf[b++] = t;
		t = getbyte();
		if (b >= BSIZE)
		    error("Compilererror [lexan]: Number to big");
	    }
	    if ((sscanf(lexbuf,"%d",&tokenval)) != 1)
		error("Compilererror [lexan]: Wrong number");

	    if (t != EOS)
		ungetb(t);

	    return (NUM);
	}
	else if (isalpha(t)) {
	    while (isalnum(t)) {
		lexbuf[b++] = t;
		t = getbyte();
		if (b >= BSIZE)
		    error("Compilerfehler: lexan");
	    }
	    lexbuf[b] = EOS;

	    p = lookup(lexbuf);
	    if (p == 0) {
		if (t == '(')
		    p = insert(lexbuf,FC);
		else
		    p = insert(lexbuf,ID);
	    }

	    if (t != EOS)
		ungetb(t);

	    tokenval = p;
	    return (symtable[p].token);
	}
	else if (t == EOS)
	    return DONE;
	else {
	    tokenval = NONE;
	    return (t);
	}
    }
}

int lookahead;

void parse(void)
{
    lookahead = lexan();
    while (lookahead != DONE) {
	expr();
	/* match(';'); */
    }
}

void expr(void)
{
    int t;
    term();
    while (TRUE) {
	switch (lookahead) {
	    case '+':
	    case '-':
		t = lookahead;
		match (lookahead);
		term();
		emit(t,NONE);
		continue;
	    default:
		return;
	}
    }
}

void term(void)
{
    int t;
    pow();
    while (TRUE) {
	switch (lookahead) {
	    case '*':
	    case '/':
	    case DIV:
	    case MOD:
		t = lookahead;
		match(lookahead);
		pow();
		emit(t,NONE);
		continue;
	    default:
		return;
	}
    }
}

void pow(void)
{
    int t;
    factor();
    while (TRUE) {
	if (lookahead == '^') {
	    t = lookahead;
	    match(lookahead);
	    factor();
	    emit(t,NONE);
	    continue;
	}
	else return;
    }
}

void factor(void)
{
    int t;

    switch (lookahead) {
	case '(':
	    match('(');
	    expr();
	    match(')');
	    break;
	case FC:
	    t = tokenval;
	    match(FC);
	    factor();
	    emit(FC,t);
	    break;
	case NUM:
	    emit(NUM,tokenval);
	    match(NUM);
	    break;
	case ID:
	    emit(ID,tokenval);
	    match(ID);
	    break;
	default:
	    error("Syntaxfehler: factor");
    }
}

void match(int t)
{
    if (lookahead == t)
	lookahead = lexan();
    else
	error("Syntaxfehler: match");
}

void emit(int t, int tval)
{
    switch (t) {
	case '+':
	    printf("\tADD\n");
	    break;
	    
	case '-':
	    printf("\tSUB\n");
	    break;
	    
	case '*':
	    printf("\tMUL\n");
	    break;
	    
	case '/':
	    printf("\tDIV\n");
	    break;
	    
	case '^':
	    printf("\tPOW\n");
	    break;

	case NUM:
	    printf("\tPUSH\t%d\n",tval);
	    break;
	    
	case ID:
	    printf("\tPUSH\t_%s\n",symtable[tval].lexptr);
/*				
	    if (strcmp(symtable[tval].lexptr,"x") == 0) {
		printf("\tPUSH\t_%s",);
	    }
	    else {
		error("Unsuported variable");
		printf("\tmove.l\t%s,-(sp)\n",symtable[tval].lexptr);
	    }
*/
	    break;
	    	    
	case FC:
	    printf("\tCALL\t_%s\n",symtable[tval].lexptr);
	    break;
	default:
	    printf("token %d, tokenval %d\n",t,tval);
    }
}

#define STRMAX 999
#define SYMMAX 100

char lexemes[STRMAX];
int  lastchar = -1;
struct entry symtable[SYMMAX];
int lastentry = 0;

int lookup(char s[])
{
    int p;
    for (p = lastentry; p>0; p--)
	if (strcmp(symtable[p].lexptr,s) == 0)
	    return p;
    return (0);
}

int insert(char s[], int tok)
{
    int len;
    len = strlen(s);
    if (lastentry+1 >= SYMMAX)
	error("Symboltabelle voll");
    if (lastchar + len + 1 >= STRMAX)
	error("Lexemarry voll");
    lastentry += 1;
    symtable[lastentry].token = tok;
    symtable[lastentry].lexptr = &lexemes[lastchar + 1];
    lastchar += len + 1;
    strcpy(symtable[lastentry].lexptr,s);
    return (lastentry);
}

char line[200];
int  pos = 0;

int error(char *m)
{
    printf("%s\n",m);
    printf("Pos: %d, str: %s\n",pos,line);
    SymOut();
    printf("Lookahead: %d\n",lookahead);
    exit(1);
}


int getbyte(void)
{
    return((int) line[pos++]);
}

void ungetb(int t)
{
    line[--pos] = (char) t;
}

void SymOut(void)
{
    int i;

    printf("\nSymtable: %d entries  Stringtable: %d chars\n",lastentry,lastchar+1);

    for (i=1; i<=lastentry; i++)
	printf("Entry %d:  Cont %s:  Token %d\n",i,symtable[i].lexptr,symtable[i].token);
}

int main(void)
{
    printf("\nThis is a simple math-function-compiler Version 0.1\n");
    printf("Written and (c) 1992 by Klaus Rotter\n");
    printf("Enter the function\n\nf(x)= ");
    fflush(stdout);
    gets(line);
    line[199]=0;
    pos = 0;
    printf("\n");
    parse();
    SymOut();
    exit(0);
}
