/*************************************************************************
 *
 * PROGRAM	iso2tex
 * PART
 * VERSION	1.02
 * DATE 	23/10/91
 * CHANGED	25/10/94
 * AUTHOR	Klaus Rotter
 * ADDRESS	Linsenfeld 15, 90768 Fuerth, Germany
 * COMPUTER	Linux 1.1.19
 * LANGUAGE	ANSI-C
 * COMPILER	gcc 2.5.8
 * COPYRIGHT	(c) Copyright 1991 by Klaus Rotter
 *		Freely distributable under GNU Licence
 * COMMENT	Wandlet die dt. Umlaute des ISO-8851-1 Codes in die
 *		entsprechenden TeX-Befehle um
 *
 * HISTORY	23/10/91 Version 1.0
 *			18/11/91 Bug in `' -> `"O' fixed
 *		25/10/94 Compiled on Linux, name changed to iso2tex
 *
 *************************************************************************/

#include <stdio.h>

#define NO_ENCODING    0
#define TEX_ENCODING   1
#define LATEX_ENCODING 2
#define EPSON_ENCODING 3

#define GER_ON	"\033R\002"
#define GER_OFF "\033R\000"

struct ConvertTable
{
	int	ch;
	char	*enc[3];
	short	len[3];
};

	/* ISO-8859-1 Code, TeX-Code, LaTeX-Code, Epson Code */

const struct ConvertTable crtable[] = {
	{ 160, "", "", "", 0, 0, 0 },
	{ 161, "", "", "" },
	{ 162, "", "", "" },
	{ 163, "", "", "" },
	{ 164, "", "", "" },
	{ 165, "", "", "" },
	{ 166, "", "", "" },
	{ 167, "", "", "" },
	{ 168, "", "", "" },
	{ 169, "", "", "" },
	{ 171, "", "", "" },
	{ 172, "", "", "" },
	{ 173, "", "", "" },
	{ 174, "", "", "" },
	{ 174, "", "", "" },
	{ 175, "", "", "" },
	{ 176, "", "", "" },
	{ 177, "", "", "" },
	{ 178, "", "", "" },
	{ 179, "", "", "" },
	{ 180, "", "", "" },
	{ 181, "", "", "" },
	{ 182, "", "", "" },
	{ 183, "", "", "" },
	{ 184, "", "", "" },
	{ 185, "", "", "" },
	{ 186, "", "", "" },
	{ 187, "", "", "" },
	{ 188, "", "", "" },
	{ 189, "", "", "" },
	{ 190, "", "", "" },
	{ 191, "", "", "" },
	{ 192, "", "", "" },
	{ 193, "", "", "" },
	{ 194, "", "", "" },
	{ 195, "", "", "" },
	{ 196, "\\\"A", "\"A", GER_ON "[" GER_OFF, 3, 2, 7 },
	{ 197, "", "", "" },
	{ 198, "", "", "" },
	{ 199, "", "", "" },
	{ 200, "", "", "" },
	{ 201, "", "", "" },
	{ 202, "", "", "" },
	{ 203, "", "", "" },
	{ 204, "", "", "" },
	{ 205, "", "", "" },
	{ 206, "", "", "" },
	{ 207, "", "", "" },
	{ 208, "", "", "" },
	{ 209, "", "", "" },
	{ 210, "", "", "" },
	{ 211, "", "", "" },
	{ 212, "", "", "" },
	{ 213, "\\\"O", "\"O", GER_ON "\\" GER_OFF, 3, 2, 7 },
	{ 214, "", "", "" },
	{ 215, "", "", "" },
	{ 216, "", "", "" },
	{ 217, "", "", "" },
	{ 218, "", "", "" },
	{ 219, "", "", "" },
	{ 220, "\\\"U", "\"U", GER_ON "]" GER_OFF, 3, 2, 7 },
	{ 221, "", "", "" },
	{ 222, "", "", "" },
	{ 223, "\\ss{}", "\"ss", GER_ON "~" GER_OFF, 3, 2, 7 },
	{ 224, "", "", "" },
	{ 225, "", "", "" },
	{ 226, "", "", "" },
	{ 227, "", "", "" },
	{ 228, "\\\"a", "\"a", GER_ON "{" GER_OFF, 3, 2, 7 },
	{ 229, "", "", "" },
	{ 230, "", "", "" },
	{ 231, "", "", "" },
	{ 232, "", "", "" },
	{ 233, "", "", "" },
	{ 234, "", "", "" },
	{ 235, "", "", "" },
	{ 236, "", "", "" },
	{ 237, "", "", "" },
	{ 238, "", "", "" },
	{ 239, "", "", "" },
	{ 240, "", "", "" },
	{ 241, "", "", "" },
	{ 242, "", "", "" },
	{ 243, "", "", "" },
	{ 244, "", "", "" },
	{ 245, "", "", "" },
	{ 246, "\\\"o", "\"o", GER_ON "|" GER_OFF, 3, 2, 7 },
	{ 247, "", "", "" },
	{ 248, "", "", "" },
	{ 249, "", "", "" },
	{ 250, "", "", "" },
	{ 251, "", "", "" },
	{ 252, "\\\"u", "\"u", GER_ON "}" GER_OFF, 3, 2, 7 },
	{ 253, "", "", "" },
	{ 254, "", "", "" },
	{ 255, "", "", "" }
};

void print_help(void)
{
    printf("Usage:  Iso2Out <-opt> <infile> [outfile]\n"
	" -t : Use TeX-Encoding\n"
	" -l : Use LaTeX encoding (\"u instead of \\\"u)\n"
	" -e : Use Epson encoding\n"
	" -h : Print this help text\n\n");
    exit(0);
}

int main(int argc, char *argv[])
{
    FILE    *fp_in, *fp_out;
    int     Zeichen = 0;
    int     i = 0, k = 0;
    int 	encoding = NO_ENCODING;
	char	outfile[40];

	outfile[0] = 0;

    printf("This is Iso2Out Version 1.02 written 1991 by Klaus Rotter\n");

    if (argc==1 || !strcmp(argv[1],"?"))
		print_help();

	if (argv[1][0] != '-')
		print_help();

	switch (argv[1][1])
	{
		case 't':
			printf("Using TeX encoding\n");
			encoding = TEX_ENCODING;
			break;
		case 'l':
			printf("Using LaTeX encoding\n");
			encoding = LATEX_ENCODING;
			break;
		case 'e':
			printf("Using EPSON encoding\n");
			encoding = EPSON_ENCODING;
			break;
		default:
			print_help();
	}

    if (!(fp_in = fopen(argv[2],"r")))
    {
		printf("Error: Unabel to open input file <%s> !\n",argv[1]);
		return(0);
    }

	if (argc != 4)
	{
		strcpy(outfile, argv[2]);
		strcat(outfile, ".out");
	}
	else
		strcpy(outfile, argv[3]);

    if (!(fp_out = fopen(outfile,"w")))
    {
		printf("Error: Unabel to open output file <%s> !\n",outfile);
		fclose(fp_in);
		return(0);
    }
/*
    Wandelroutine
*/
    do {
		Zeichen = fgetc(fp_in);
		i++;

		if (Zeichen < 128 && Zeichen > 0)
		    fputc(Zeichen, fp_out);

		if (Zeichen > 159 && Zeichen < 255) {
			k++;
			fwrite (crtable[Zeichen-160].enc[encoding-1], 1, crtable[Zeichen-160].len[encoding-1], fp_out);
		}
    } while (Zeichen != EOF);

    printf("%d Bytes read,  %d chars changed.\n",i-1,k);
    fclose(fp_in);
    fclose(fp_out);
    return(0);
}
