/*************************************************************************
 *
 * PROGRAM	AvalonMan
 * PART
 * VERSION	V0.1
 * DATE 	12/23/94
 * CHANGED
 * AUTHOR	Klaus Rotter
 * ADDRESS	Linsenfeld 15, 90768 Fuerth, Germany
 * EMAIL	rotter@mi.uni-erlangen.de
 * COMPUTER	Linux
 * EXTENSION
 * LANGUAGE	ANSI-C
 * COMPILER	GCC 2.5.8
 * COPYRIGHT	Public Domain
 * COMMENT
 * HISTORY
 *
 *************************************************************************/

#define MAX_X 640
#define MAX_Y 512
#define MAX_FARBEN 31

#include <math.h>
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/xv_xrect.h>
#include <xview/panel.h>
#include <xview/frame.h>

static short color[] = {
    0x000, 0x03F, 0x05F, 0x08F, 0x0AF, 0x0DF, 0x0FE, 0x0F8, 0x0F6,
    0x0F3, 0x0F0, 0x3F0, 0x5F0, 0x8F0, 0x0F0, 0x2F0, 0x5F0, 0x5C0,
    0x5A0, 0x590, 0xF50, 0xF20, 0xF00, 0xF04, 0xF08, 0xF09, 0xF0C,
    0xF0F, 0xB0F, 0x70F, 0x40F, 0x00F
};

extern void quit (void);
extern void paint ();

Frame frame;
Panel panel;
Canvas canvas;

float min_x, min_y, max_x, max_y;
int   max_it;

void quit(void)
{
    xv_destroy_safe (frame);
    exit(0);
}

void main (int argc, char *argv[])
{
    printf("Berechnung der Mandelbrot-Menge (Apfelmnchen)\n\n");

    printf("Startwert X-Koordinate: "); fflush(stdout);
    scanf("%f",&min_x);

    printf("Startwert Y-Koordinate: "); fflush(stdout);
    scanf("%f",&min_y);

    printf("Endwert X-Koordinate: "); fflush(stdout);
    scanf("%f",&max_x);

    printf("Entwert Y-Koordinate: "); fflush(stdout);
    scanf("%f",&max_y);

    printf("Wieviele Iterationen: "); fflush(stdout);
    scanf("%d",&max_it);

    xv_init (XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

    frame = (Frame) xv_create (XV_NULL, FRAME, NULL);

    panel = (Panel) xv_create (frame, PANEL, PANEL_LAYOUT, PANEL_VERTICAL, NULL);

    xv_create (panel, PANEL_BUTTON,
	PANEL_LABEL_STRING,	"Quit",
	PANEL_NOTIFY_PROC,	quit,
	NULL);

    xv_create (panel, PANEL_BUTTON,
	PANEL_LABEL_STRING,	"Paint",
	PANEL_NOTIFY_PROC,	paint,
	NULL);

    window_fit_width (panel);

    canvas = (Canvas) xv_create (frame, CANVAS,
	CANVAS_REPAINT_PROC,	paint,
	CANVAS_X_PAINT_WINDOW,	TRUE,
	CANVAS_WIDTH,		640,
	CANVAS_HEIGHT,		512,
	NULL);

    xv_main_loop (frame);
}

void paint (void)
{
    short   zeile, spalte, farbe;
    float   cr, ci;
    float   zr, zi, i;
    float   delta_x, delta_y;
    short   it;

    GC		gc;
    Window	xwin;
    Display	*dpy;

    xwin = (Window) xv_get (canvas_paint_window (canvas), XV_XID);
    dpy = (Display *) xv_get (frame, XV_DISPLAY);

    gc = DefaultGC (dpy, DefaultScreen(dpy));

    XClearWindow (dpy, xwin);

    cr = min_x;
    ci = min_y;

    delta_x = (max_x - min_y) / MAX_X;
    delta_y = (max_y - min_y) / MAX_Y;

    printf("%f %f %f %f\n%f %f\n",min_x, min_y, max_x, max_y, delta_x, delta_y);

    for (zeile=0; zeile <= MAX_Y; zeile++)
    {
	for (spalte=0; spalte <= MAX_X; spalte++)
	{
	    it = 0;
	    zr = 0;
	    zi = 0;

	    while ((it < max_it) && (fabs(sqrt(zr*zr + zi*zi)) < 4))
	    {
		i = zi;
		zi = 2*i*zr-ci;
		zr = zr*zr-i*i-cr;
		it++;
	    }

	    farbe = (it % MAX_FARBEN)+1;

	    if (it >= max_it) farbe = 0;

	    XSetForeground(dpy, gc, farbe);

	    XDrawPoint (dpy, xwin, gc, zeile, spalte);

	    cr += delta_x;
	}
	cr = min_x;
	ci += delta_y;
    }
}
