/*
 * Programm:	PI.c
 * Datei:	---
 * Version:	V1.0
 * Datum:	02.08.91
 * l. nderung: ---
 * Autor:	Public Domain, see below
 * Computer:	Amiga 500 mit Kickstart 1.3
 * Erweiterung:
 * Sprache:	Ansi-C
 * Compiler:	DICE 2.06
 * Copyright:	It's in the Public Domain (see below)
 * Bemerkung:
 */

/*
 * Program Pi.c originally distributed by Microsoft to demonstarate Code
 * view  for IBM and compatibles.
 * Released by Microsoft into the public domain
 */

#include <time.h>
#include <stdio.h>

#define  MAXPRC 400000
#define  FALSE	0
#define  TRUE	1

/*
 *  Prototypes
 */

void arctan(int s);
void add(void);
void sub(void);
void mul(int multiplier);
void div(int divisor);
void div4(void);
void mul4(void);
int  tiszero(void);

char	p[MAXPRC];
char	t[MAXPRC];
int	q;

void main(int argc,char *argv[])
{
   long startime, endtime;
   int i;

   if (argc == 2) {
      sscanf(argv[1],"%d",&q);
   } else {
      printf("Usage: pi [precision]\n");
      exit(-1);
   }

   /* compute PI */

   time(&startime);
   arctan(2);
   arctan(3);
   mul4();
   time(&endtime);

   /* print PI */

   printf("PI = %d.",p[0]);
   for ( i = 1; i <= q; i++)
      printf("%d",p[i]);
   printf("\n");
   printf("%ld seconds to compute %d digits of PI\n",endtime - startime,q);
}

void arctan(int s)
{
   int n;

   t[0] = 1;
   div(s);              /* t[] = 1/s */
   add();
   n = 1;

   do {
	 mul(n);
	 div(s * s);
	 div(n += 2);
	 if (((n-1) / 2) % 2 == 0)
	    add();
	 else
	    sub();
      } while (!tiszero());
}

void add(void)
{
   int j;

   for (j = q; j >= 0; j--)
      if (t[j] + p[j] > 9) {
	 p[j] += t[j] - 10;
	 p[j-1] += 1;
      } else
	 p[j] += t[j];
}

void sub(void)
{
   int j;

   for (j = q; j >= 0; j--)
      if (p[j] < t[j])  {
	 p[j] -= t[j] - 10;
	 p[j-1] -= 1;
      } else
	 p[j] -= t[j];
}

void mul(int multiplier)
{
   int b;
   int i;
   int carry = 0, digit = 0;

   for (i = q; i >= 0; i--) {
      b = (t[i] * multiplier + carry);
      digit = b % 10;
      carry = b / 10;
      t[i] = digit;
   }
}

/* t[] /= 1 */

void div(int divisor)
{
   int i, b;
   int quotient, remainder = 0;

   for (i = 0; i <= q; i++) {
      b = (10 * remainder + t[i]);
      quotient = b / divisor;
      remainder = b % divisor;
      t[i] = quotient;
   }
}

void div4(void)
{
   int i, c, d = 0;

   for (i=0; i <=q; i++) {
      c = (10 * d + p[i]) / 4;
      d = (10 * d + p[i]) % 4;
      p[i] = c;
   }
}

void mul4(void)
{
   int i, c, d;

   d = c = 0;

   for (i = q; i >= 0; i--) {
      d = (p[i] * 4 + c) % 10;
      c = (p[i] * 4 + c) / 10;
      p[i] = d;
   }
}

int tiszero(void)
{
   int k;

   for (k = 0; k <= q; k++)
      if (t[k] != 0)
	 return(FALSE);
   return(TRUE);
}

